<?php

declare(strict_types=1);

namespace hood\vk\robots;

use hood\vk\robots\robot,
    hood\vk\api\longpoll;

use Throwable,
    Exception;

/**
 * Робот-группа
 *
 * $longpoll LongPoll-сессия
 *
 * public function __set($name, $value) Запись свойства
 * public function __get($name) Чтение свойства
 * public function __isset($name) Проверка на инициализированность свойства
 *
 * hood\vk\robots
 * Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 */
final class group extends robot
{
    /**
     * $longpoll LongPoll-сессия
     */
    protected longpoll $longpoll;

    /**
     * Запись свойства
     *
     * $name Название
     * $value Значение
     *
     * @see hood\vk\robots\robot Наследуемый метод
     */
    public function __set(string $name, mixed $value): void
    {
        try {
            parent::__set($name, $value);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            match ($name) {
                'longpoll' => $value instanceof LongPoll ? $this->longpoll = $value : $this->longpoll = new LongPoll($this),
                default => throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious())
            };
        }
    }

    /**
     * Чтение свойства
     *
     * $name Название
     *
     * @see hood\vk\robots\robot Наследуемый метод
     */
    public function __get(string $name): mixed
    {
        try {
            return parent::__get($name);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            if ($name === 'longpoll') {
                if (!isset($this->longpoll)) {
                    $this->longpoll = new LongPoll($this);
                }
                return $this->longpoll;
            }

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }

    /**
     * Проверка на инициализированность свойства
     *
     * $name Название
     *
     * @return mixed
     */
    public function __isset(string $name)
    {
        try {
            return parent::__isset($name);
        } catch (Throwable $e) {
            // Если свойство не найдено в родительском методе

            if ($name === 'longpoll') {
                return isset($this->longpoll);
            }

            throw new Exception($e->getMessage(), $e->getCode(), $e->getPrevious());
        }
    }
}
